from pathlib import Path

from click import make_pass_decorator
from pydantic import BaseModel

from delfino.constants import PackageManager
from delfino.models.pyproject_toml import PyprojectToml


class AppContext(BaseModel):
    project_root: Path
    py_project_toml: PyprojectToml
    package_manager: PackageManager

    class Config:
        arbitrary_types_allowed = True


pass_app_context = make_pass_decorator(AppContext)



"""Tests on source code."""

import re
import shutil
import webbrowser
from pathlib import Path
from subprocess import PIPE

import click

from delfino.contexts import AppContext, pass_app_context
from delfino.execution import OnError, run
from delfino.terminal_output import print_header, run_command_example
from delfino.utils import ensure_reports_dir


def _run_tests(app_context: AppContext, name: str, maxfail: int, debug: bool) -> None:
    """Execute the tests for a given test type."""
    delfino = app_context.py_project_toml.tool.delfino

    if name not in delfino.test_types or not delfino.tests_directory:
        return

    print_header(f"️Running {name} tests️", icon="🔎🐛")
    ensure_reports_dir(delfino)
    run(
        [
            "pytest",
            "--cov",
            delfino.sources_directory,
            "--cov-report",
            f"xml:{delfino.reports_directory / f'coverage-{name}.xml'}",
            "--cov-branch",
            "-vv",
            "--maxfail",
            str(maxfail),
            "-s" if debug else "",
            delfino.tests_directory / name,
        ],
        env_update={"COVERAGE_FILE": delfino.reports_directory / f"coverage-{name}.dat"},
        on_error=OnError.ABORT,
    )


@click.command(help="Run unit tests.")
@click.option("--maxfail", type=int, default=0)
@click.option("--debug", is_flag=True, help="Disables capture, allowing debuggers like `pdb` to be used.")
@pass_app_context
def test_unit(app_context: AppContext, maxfail: int, debug: bool):
    _run_tests(app_context, "unit", maxfail=maxfail, debug=debug)


@click.command(help="Run integration tests.")
@click.option("--maxfail", type=int, default=0)
@click.option("--debug", is_flag=True, help="Disables capture, allowing debuggers like `pdb` to be used.")
@pass_app_context
def test_integration(app_context: AppContext, maxfail: int, debug: bool):
    # TODO(Radek): Replace with alias?
    _run_tests(app_context, "integration", maxfail=maxfail, debug=debug)


def _get_total_coverage(coverage_dat: Path) -> str:
    """Return coverage percentage, as captured in coverage dat file; e.g., returns "100%"."""
    output = run(
        "coverage report", stdout=PIPE, env_update={"COVERAGE_FILE": coverage_dat}, on_error=OnError.EXIT
    ).stdout.decode()
    match = re.search(r"TOTAL.*?([\d.]+%)", output)
    if match is None:
        raise RuntimeError(f"Regex failed on output: {output}")
    return match.group(1)


@click.command()
@pass_app_context
def coverage_report(app_context: AppContext):
    """Analyse coverage and generate a term/HTML report.

    Combines all test types.
    """
    print_header("Generating coverage report", icon="📃")
    delfino = app_context.py_project_toml.tool.delfino
    ensure_reports_dir(delfino)

    coverage_dat_combined = delfino.reports_directory / "coverage.dat"
    coverage_html = delfino.reports_directory / "coverage-report/"
    coverage_files = []  # we'll make a copy because `combine` will erase them

    for test_type in delfino.test_types:
        coverage_dat = delfino.reports_directory / f"coverage-{test_type}.dat"

        if not coverage_dat.exists():
            click.secho(
                f"Could not find coverage dat file for {test_type} tests: {coverage_dat}",
                fg="yellow",
            )
        else:
            print(f"{test_type.title()} test coverage: {_get_total_coverage(coverage_dat)}")

            temp_copy = coverage_dat.with_name(coverage_dat.name.replace(".dat", "-copy.dat"))
            shutil.copy(coverage_dat, temp_copy)
            coverage_files.append(temp_copy)

    env = {"COVERAGE_FILE": coverage_dat_combined}
    run(["coverage", "combine", *coverage_files], env_update=env, stdout=PIPE, on_error=OnError.EXIT)
    run(["coverage", "html", "-d", coverage_html], env_update=env, stdout=PIPE, on_error=OnError.EXIT)

    print(f"Total coverage: {_get_total_coverage(coverage_dat_combined)}\n")
    print(
        f"Refer to coverage report for full analysis in '{coverage_html}/index.html'\n"
        f"Or open the report in your default browser with:\n"
        f"  {run_command_example(coverage_open, app_context)}"
    )


@click.command(help="Run all tests, and generate coverage report.")
@click.pass_context
def test_all(click_context: click.Context):
    print_header("Linting", icon="🔎")
    click_context.forward(test_unit)
    click_context.forward(test_integration)
    click_context.forward(coverage_report)


@click.command(help="Open coverage results in default browser.")
@pass_app_context
def coverage_open(app_context: AppContext):
    report_index = app_context.py_project_toml.tool.delfino.reports_directory / "coverage-report" / "index.html"
    if not report_index.exists():
        click.secho(
            f"Could not find coverage report '{report_index}'. Ensure that the report has been built.\n"
            "Try one of the following:\n"
            f"  {run_command_example(coverage_report, app_context)}\n"
            f"or\n"
            f"  {run_command_example(test_all, app_context)}",
            fg="red",
        )

        raise click.exceptions.Exit(code=1)
    webbrowser.open(f"file:///{report_index.absolute()}")



import click

from delfino.commands.format import run_format
from delfino.commands.lint import lint
from delfino.commands.test import test_all
from delfino.commands.typecheck import typecheck
from delfino.utils import command_names

_COMMANDS = [run_format, lint, typecheck, test_all]


@click.command(help=f"Runs all checks.\n\nAlias for the {command_names(_COMMANDS)} commands.")
@click.pass_context
def verify_all(click_context: click.Context):
    for command in _COMMANDS:
        click_context.forward(command)



from subprocess import PIPE, CompletedProcess

import click

from delfino.contexts import AppContext, pass_app_context
from delfino.execution import OnError, run
from delfino.terminal_output import print_header, run_command_example


def _check_result(app_context: AppContext, result: CompletedProcess, check: bool, msg: str):
    if result.returncode == 1 and check:
        click.secho(
            f"{msg} before commit. Try following:\n"
            f" * Enable pre-commit hook by running `pre-commit install` in the repository.\n"
            f" * Run formatter manually with `{run_command_example(run_format, app_context)}` before committing code.",
            fg="red",
            err=True,
        )
        raise click.Abort()

    if result.returncode > 1:
        raise click.Abort()


@click.command("format")
@click.option("--check", is_flag=True, help="Only check formatting, don't reformat the code.")
@click.option("--quiet", is_flag=True, help="Don't show progress. Only errors.")
@pass_app_context
def run_format(app_context: AppContext, check: bool, quiet: bool):
    """Runs black code formatter and isort on source code."""
    # ensure pre-commit is installed
    run("pre-commit install", stdout=PIPE, on_error=OnError.EXIT)

    delfino = app_context.py_project_toml.tool.delfino
    dirs = [delfino.sources_directory, delfino.tests_directory]
    flags = []

    if check:
        flags.append("--check")

    print_header("Sorting imports", icon="ℹ")

    _check_result(app_context, run(["isort", *dirs, *flags], on_error=OnError.PASS), check, "Import were not sorted")

    print_header("Formatting code", icon="🖤")

    if quiet:
        flags.append("--quiet")

    _check_result(app_context, run(["black", *dirs, *flags], on_error=OnError.PASS), check, "Code was not formatted")



from getpass import getpass
from os import getenv
from pathlib import Path
from typing import List

import click

from delfino.constants import PackageManager
from delfino.contexts import AppContext, pass_app_context
from delfino.execution import OnError, run
from delfino.terminal_output import print_header
from delfino.utils import ArgsList
from delfino.validation import assert_pip_package_installed, pyproject_toml_key_missing

try:
    from packaging.version import Version
except ImportError:
    pass


def _install_emulators(build_for_platforms: List[str]) -> None:
    """See https://github.com/tonistiigi/binfmt#installing-emulators."""
    emulators = []
    if "linux/arm64" in build_for_platforms:
        emulators.append("arm64")
    if "linux/arm/v7" in build_for_platforms:
        emulators.append("arm")

    if emulators:
        print_header("Installing emulators", level=2, icon="⬇")
        run(
            ["docker", "run", "--privileged", "--rm", "tonistiigi/binfmt", "--install", *emulators],
            on_error=OnError.EXIT,
        )


@click.command()
@click.option(
    "--push",
    is_flag=True,
    help="Push the built image and cache to remote docker image registry. Login is "
    "required and Personal Access Token will be requested from STDIN if "
    "`DOCKERHUB_PERSONAL_ACCESS_TOKEN` environment variable is not set.",
)
@pass_app_context
def build_docker(app_context: AppContext, push: bool):
    """Build and push a docker image."""
    delfino = app_context.py_project_toml.tool.delfino
    dockerfile = Path("Dockerfile")

    assert (
        app_context.package_manager == PackageManager.POETRY
    ), f"Only the '{PackageManager.POETRY.value}' package manager is supported by this command."
    assert_pip_package_installed("packaging")
    assert dockerfile.exists() and dockerfile.is_file(), "File 'Dockerfile' not found but required by this command."
    assert app_context.py_project_toml.tool.poetry, pyproject_toml_key_missing("tool.poetry")
    assert delfino.dockerhub, pyproject_toml_key_missing("tool.delfino.dockerhub")

    print_header("Running Docker build")

    flags: ArgsList = []

    poetry = app_context.py_project_toml.tool.poetry
    project_name = poetry.name
    project_version = poetry.version

    python_version = Version(poetry.dependencies["python"].strip("<>=~^"))
    flags.extend(["--build-arg", f"PYTHON_VERSION='{python_version.major}.{python_version.minor}'"])

    dockerhub = delfino.dockerhub
    flags.extend(["--platform", ",".join(dockerhub.build_for_platforms)])

    if push:
        dockerhub_password = getenv("DOCKERHUB_PERSONAL_ACCESS_TOKEN")
        if not dockerhub_password:
            click.secho("⚠ The 'DOCKERHUB_PERSONAL_ACCESS_TOKEN' environment variable is not set.", fg="yellow")
            dockerhub_password = getpass("Dockerhub Personal Access Token: ")
        run(f"docker login --username {dockerhub.username} --password {dockerhub_password}", on_error=OnError.EXIT)

        flags.extend(["--output", "type=image,push=true"])

    if getenv("CI"):  # https://circleci.com/docs/2.0/env-vars/#built-in-environment-variables
        flags.extend(["--progress", "plain"])

    _install_emulators(dockerhub.build_for_platforms)

    # While `docker buildx build` supports multiple `--tag` flags, push of them fails to expose
    # all architectures in `latest`. Multiple pushes fix this.
    for tag in [project_version, "latest"]:
        print_header(f"Build {dockerhub.username}/{project_name}:{tag}", level=2, icon="🔨")

        flags_for_tag = list(flags)
        if tag != "latest" and push:
            flags_for_tag.extend(["--cache-to", f"type=registry,ref={dockerhub.username}/{project_name}"])

        run(
            [
                "docker",
                "buildx",
                "build",
                "--cache-from",
                f"type=registry,ref={dockerhub.username}/{project_name}",
                "--tag",
                f"{dockerhub.username}/{project_name}:{tag}",
                *flags_for_tag,
                ".",
            ],
            on_error=OnError.EXIT,
        )

    if push:
        run("docker logout", on_error=OnError.EXIT)



import os
import sys
from pathlib import Path
from typing import Any, Dict, List, Optional, Set, Union

import click
import toml
from pydantic import ValidationError

from delfino import commands
from delfino.constants import ENTRY_POINT, PYPROJECT_TOML_FILENAME
from delfino.contexts import AppContext
from delfino.models.pyproject_toml import PyprojectToml
from delfino.utils import find_commands, get_package_manager


class Commands(click.MultiCommand):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        sys.path.append(os.getcwd())

        self._project_root = Path(os.getcwd())
        pyproject_toml_path = (self._project_root / PYPROJECT_TOML_FILENAME).relative_to(self._project_root)

        # When evaluating available commands, the program should not fail. We save the exception
        # to show it only when a command is executed.
        self._pyproject_toml: Union[PyprojectToml, Exception]
        self._hidden_plugins: Set[str] = set()

        try:
            self._pyproject_toml = PyprojectToml(file_path=pyproject_toml_path, **toml.load(pyproject_toml_path))
            self._hidden_plugins = self._pyproject_toml.tool.delfino.disable_commands
        except (FileNotFoundError, ValidationError) as exc:
            self._pyproject_toml = exc

        self._plugins: Dict[str, click.Command] = find_commands(commands.__package__, required=True)
        self._plugins.update(find_commands("commands", required=False))
        self._plugins.update(find_commands("tasks", required=False, new_name="commands"))

    def list_commands(self, ctx: click.Context) -> List[str]:
        """Override to hide commands marked as hidden in the ``pyproject.toml`` file."""
        del ctx
        return sorted(set(self._plugins.keys()).difference(self._hidden_plugins))

    def get_command(self, ctx: click.Context, cmd_name: str) -> Optional[click.Command]:
        """Override to give all commands a common ``AppContext`` or fail if ``pyproject.toml`` is broken/missing."""
        if isinstance(self._pyproject_toml, Exception):
            click.secho(f"Delfino appears to be misconfigured: {self._pyproject_toml}", fg="red", err=True)
            raise click.Abort() from self._pyproject_toml

        ctx.obj = AppContext(
            project_root=self._project_root,
            py_project_toml=self._pyproject_toml,
            package_manager=get_package_manager(self._project_root, self._pyproject_toml),
        )

        return self._plugins[cmd_name]

    def invoke(self, ctx: click.Context) -> Any:
        """Override to turn ``AssertionError`` exception into ``click.exceptions.Exit``."""
        try:
            return super().invoke(ctx)
        except AssertionError as exc:
            click.secho(f"Command '{ctx.invoked_subcommand}' is misconfigured. {exc}", fg="red", err=True)
            raise click.exceptions.Exit(1)

    def get_help(self, *args, **kwargs) -> str:
        help_str = super().get_help(*args, **kwargs)

        if self._hidden_plugins:
            help_str += (
                f"\n\nDisabled command{'s' if len(self._hidden_plugins) > 1 else ''}: "
                + ", ".join(sorted(self._hidden_plugins))
                + f" (see 'tool.{ENTRY_POINT}.disable_commands' in '{PYPROJECT_TOML_FILENAME}')"
            )

        return help_str


# @click.group(cls=Commands)
# def main():
#     context = click.get_current_context()

main = Commands()

if __name__ == "__main__":
    main()



"""Type checking on source code."""

import click

from delfino.contexts import AppContext, pass_app_context
from delfino.execution import OnError, run
from delfino.terminal_output import print_header
from delfino.utils import ArgsList, ensure_reports_dir


@click.command()
@click.option("--summary-only", is_flag=True, help="Suppress error messages and show only summary error count.")
@pass_app_context
def typecheck(app_context: AppContext, summary_only: bool):
    """Run type checking on source code.

    A non-zero return code from this task indicates invalid types were discovered.
    """
    print_header("RUNNING TYPE CHECKER", icon="🔠")

    delfino = app_context.py_project_toml.tool.delfino
    reports_file = delfino.reports_directory / "typecheck" / "junit.xml"

    ensure_reports_dir(delfino)

    args: ArgsList = [
        "mypy",
        "--show-column-numbers",
        "--show-error-codes",
        "--color-output",
        "--warn-unused-config",
        "--warn-unused-ignores",
        "--color-output",
        "--follow-imports",
        "silent",
        "--junit-xml",
        reports_file,
        delfino.sources_directory,
        delfino.tests_directory,
    ]

    if summary_only:
        args.extend(["|", "tail", "-n", "1"])

    run(args, env_update_path={"MYPYPATH": delfino.sources_directory}, shell=summary_only, on_error=OnError.ABORT)



from delfino.constants import PYPROJECT_TOML_FILENAME
from delfino.execution import pip_package_installed


def pyproject_toml_key_missing(key: str):
    return f"Key '{key}' is missing from the '{PYPROJECT_TOML_FILENAME}' file."


def assert_pip_package_installed(name: str):
    assert pip_package_installed(
        name
    ), f"Optional Python package '{name}' is required by this command but not installed."



from pathlib import Path
from typing import Any, Dict, List, Optional, Set

from pydantic import BaseModel, Extra, Field


class Dockerhub(BaseModel):
    build_for_platforms: List[str] = Field(..., min_items=1)
    username: str


class Delfino(BaseModel):
    sources_directory: Path
    tests_directory: Path
    reports_directory: Path
    test_types: List[str]
    disable_commands: Set[str] = Field(default_factory=set)

    dockerhub: Optional[Dockerhub] = None

    plugins: Dict[str, Any] = Field(default_factory=dict, description="Any additional config given by plugins.")

    class Config:
        extra = Extra.allow


class Poetry(BaseModel):
    name: str
    version: str
    scripts: Dict[str, str] = Field(default_factory=dict)
    dependencies: Dict[str, Any] = Field(default_factory=dict)

    class Config:
        extra = Extra.allow


class Tool(BaseModel):
    poetry: Optional[Poetry] = None
    delfino: Delfino

    class Config:
        allow_population_by_field_name = True


class PyprojectToml(BaseModel):
    file_path: Optional[Path] = None
    tool: Tool

    class Config:
        extra = Extra.allow



from importlib import import_module, resources
from importlib.resources import Package
from pathlib import Path
from typing import Dict, List, Union

import click

from delfino.constants import PackageManager
from delfino.models.pyproject_toml import Delfino, PyprojectToml

ArgsList = List[Union[str, bytes, Path]]
ArgsType = Union[str, bytes, List]


def get_package_manager(project_root: Path, py_project_toml: PyprojectToml) -> PackageManager:
    if py_project_toml.tool.poetry is not None or (project_root / "poetry.lock").exists():
        return PackageManager.POETRY
    if (project_root / "Pipfile").exists() or (project_root / "Pipfile.lock").exists():
        return PackageManager.PIPENV

    click.secho(
        "Cannot determine package manager used in this project. Only the following ones are supported: "
        + ", ".join(member.value for member in PackageManager.__members__.values()),
        fg="red",
        err=True,
    )
    raise click.Abort()


def ensure_reports_dir(delfino: Delfino) -> None:
    """Ensures that the reports directory exists."""
    delfino.reports_directory.mkdir(parents=True, exist_ok=True)


def command_names(commands: List[click.Command]) -> str:
    return ", ".join(command.name for command in commands if command.name)


def find_commands(package: Package, *, required: bool, new_name: str = "") -> Dict[str, click.Command]:
    """Finds all instances of ``click.Command`` in given module.

    Does not traverse modules recursively.

    Args:
        package: Package to search.
        new_name: If set, ``package`` is a legacy name and ``new_name`` should be used instead.
        required: This package is required.
    """
    try:
        files = resources.contents(package)
    except ModuleNotFoundError:
        if required:
            raise
        return {}

    commands: Dict[str, click.Command] = {}

    for filename in files:
        if filename.startswith("_") or not filename.endswith(".py"):
            continue
        plugin = import_module(f"{package}.{filename[:-3]}")

        for obj in vars(plugin).values():
            if isinstance(obj, click.Command) and obj.name is not None:
                commands[obj.name] = obj

    if commands and new_name:
        click.secho(f"⚠ Plugin module '{package}' is deprecated. Please use '{new_name}' instead.", fg="yellow")

    return commands



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['delfino', 'delfino.commands', 'delfino.models']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0', 'pydantic>=1.8.2,<2.0.0', 'toml>=0.10.2,<0.11.0']

extras_require = \
{'all': ['black',
         'isort',
         'pre-commit',
         'pytest',
         'coverage',
         'pytest-cov',
         'pytest-dotenv',
         'pytest-mock',
         'mypy',
         'pylint',
         'pycodestyle',
         'pydocstyle',
         'twine'],
 'format': ['black', 'isort', 'pre-commit'],
 'lint': ['pylint', 'pycodestyle', 'pydocstyle'],
 'test': ['pytest', 'coverage', 'pytest-cov', 'pytest-dotenv', 'pytest-mock'],
 'typecheck': ['mypy'],
 'upload_to_pypi': ['twine'],
 'verify_all': ['black',
                'isort',
                'pre-commit',
                'pytest',
                'coverage',
                'pytest-cov',
                'pytest-dotenv',
                'pytest-mock',
                'mypy',
                'pylint',
                'pycodestyle',
                'pydocstyle']}

entry_points = \
{'console_scripts': ['delfino = delfino.main:main', 'mike = delfino.main:main']}

setup_kwargs = {
    'name': 'delfino',
    'version': '0.8.0',
    'description': 'A collection of command line helper scripts wrapping tools used during Python development.',
    'long_description': '<h1 align="center" style="border-bottom: none;">🧰&nbsp;&nbsp;Delfino&nbsp;&nbsp;🧰</h1>\n<h3 align="center">A collection of command line helper scripts wrapping tools used during Python development.</h3>\n\n<p align="center">\n    <a href="https://app.circleci.com/pipelines/github/radeklat/delfino?branch=main">\n        <img alt="CircleCI" src="https://img.shields.io/circleci/build/github/radeklat/delfino">\n    </a>\n    <a href="https://app.codecov.io/gh/radeklat/delfino/">\n        <img alt="Codecov" src="https://img.shields.io/codecov/c/github/radeklat/delfino">\n    </a>\n    <a href="https://github.com/radeklat/delfino/tags">\n        <img alt="GitHub tag (latest SemVer)" src="https://img.shields.io/github/tag/radeklat/delfino">\n    </a>\n    <img alt="Maintenance" src="https://img.shields.io/maintenance/yes/2021">\n    <a href="https://github.com/radeklat/delfino/commits/main">\n        <img alt="GitHub last commit" src="https://img.shields.io/github/last-commit/radeklat/delfino">\n    </a>\n</p>\n\n<!--\n    How to generate TOC from PyCharm:\n    https://github.com/vsch/idea-multimarkdown/wiki/Table-of-Contents-Extension\n-->\n[TOC levels=1,2 markdown formatted bullet hierarchy]: # "Table of content"\n\n# Table of content\n- [Installation](#installation)\n- [Usage](#usage)\n\n# Installation\n\nTODO\n\n# Usage\n\nTODO\n\n<!--\n\n## Minimal plugin\n\n```python\nimport click\n\nfrom delfino.contexts import pass_app_context, AppContext\n\n\n@click.command()\n@pass_app_context\ndef plugin_test(app_context: AppContext):\n    """Tests commands placed in the `commands` folder are loaded."""\n    print(app_context.py_project_toml.tool.delfino.plugins)\n```\n\n -->\n\n<!--\n\n# Install completions\n\nBased on [Click documentation](https://click.palletsprojects.com/en/8.0.x/shell-completion/?highlight=completions#enabling-completion) and Invoke implementation of dynamic completion:\n\n```bash\n# Invoke tab-completion script to be sourced with Bash shell.\n# Known to work on Bash 3.x, untested on 4.x.\n\n_complete_invoke() {\n    local candidates\n\n    # COMP_WORDS contains the entire command string up til now (including\n    # program name).\n    # We hand it to Invoke so it can figure out the current context: spit back\n    # core options, task names, the current task\'s options, or some combo.\n    candidates=`invoke --complete -- ${COMP_WORDS[*]}`\n\n    # `compgen -W` takes list of valid options & a partial word & spits back\n    # possible matches. Necessary for any partial word completions (vs\n    # completions performed when no partial words are present).\n    #\n    # $2 is the current word or token being tabbed on, either empty string or a\n    # partial word, and thus wants to be compgen\'d to arrive at some subset of\n    # our candidate list which actually matches.\n    #\n    # COMPREPLY is the list of valid completions handed back to `complete`.\n    COMPREPLY=( $(compgen -W "${candidates}" -- $2) )\n}\n\n\n# Tell shell builtin to use the above for completing our invocations.\n# * -F: use given function name to generate completions.\n# * -o default: when function generates no results, use filenames.\n# * positional args: program names to complete for.\ncomplete -F _complete_invoke -o default invoke inv\n```\n\nPut into `~/.bashrc`:\n\n```bash\n_complete_delfino() {\n    eval "$(_DELFINO_COMPLETE=bash_source delfino)";\n}\ncomplete -F _complete_delfino -o default invoke delfino\n```\n\n-->',
    'author': 'Radek Lát',
    'author_email': 'radek.lat@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/radeklat/delfino',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)
